<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2009 Pixware SARL. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind DITA Converter project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:rx="http://www.renderx.com/XSL/Extensions"
                xmlns:mml="http://www.w3.org/1998/Math/MathML"
                xmlns:svg="http://www.w3.org/2000/svg"
                xmlns:ditac="http://www.xmlmind.com/ditac/schema/ditac"
                exclude-result-prefixes="xs ditac"
                version="2.0">

  <!-- dita ============================================================== -->

  <xsl:template match="dita">
    <xsl:apply-templates/>
  </xsl:template>

  <!-- draft-comment ===================================================== -->

  <xsl:attribute-set name="draft-comment">
    <xsl:attribute name="font-size">90%</xsl:attribute>
    <xsl:attribute name="color">red</xsl:attribute>
    <!-- No margins. -->
  </xsl:attribute-set>

  <xsl:attribute-set name="draft-comment-info">
    <xsl:attribute name="font-weight">bold</xsl:attribute>
  </xsl:attribute-set>

  <xsl:template match="*[contains(@class,' topic/draft-comment ')]">
    <xsl:if test="$show-draft-comments = 'yes'">
      <fo:block xsl:use-attribute-sets="draft-comment">
        <xsl:call-template name="commonAttributes"/>

        <xsl:if test="@author or @time or @disposition">
          <fo:inline xsl:use-attribute-sets="draft-comment-info">
            <xsl:value-of
                select="string-join((@time, @author, @disposition), ', ')"/>
            <xsl:text>: </xsl:text>
          </fo:inline>
        </xsl:if>

        <xsl:apply-templates/>
      </fo:block>
    </xsl:if>
  </xsl:template>

  <!-- fn ================================================================ -->

  <xsl:template match="*[contains(@class,' topic/fn ')]">
    <xsl:if test="not(@id)">
      <fo:footnote>
        <xsl:call-template name="footnoteCallout">
          <xsl:with-param name="footnote" select="."/>
        </xsl:call-template>

        <xsl:call-template name="footnoteBody">
          <xsl:with-param name="footnote" select="."/>
        </xsl:call-template>
      </fo:footnote>
    </xsl:if>
  </xsl:template>

  <!-- indexterm ========================================================= -->

  <xsl:template match="*[contains(@class,' topic/indexterm ')]">
    <fo:inline id="{@id}" hyphenate="false">
      <xsl:call-template name="xepIndexAnchor">
        <xsl:with-param name="id" select="@id"/>
      </xsl:call-template>
    </fo:inline>
  </xsl:template>

  <xsl:template name="xepIndexAnchor">
    <xsl:param name="id" select="''"/>

    <xsl:if test="$id != '' and $foProcessor = 'XEP'">
      <xsl:variable name="anchors" 
        select="$ditacLists/ditac:indexList//ditac:indexAnchor[@id = $id or @id2 = $id]"/>
      <xsl:for-each select="$anchors">
        <xsl:variable name="key" select="../@xml:id"/>

        <xsl:choose>
          <xsl:when test="@id2 = $id">
            <rx:end-index-range 
              ref-id="{concat($key, '--', @number, '-', @number2)}"/>
          </xsl:when>
          <xsl:when test="@id2 != '' and @id = $id">
            <rx:begin-index-range rx:key="{$key}"
              id="{concat($key, '--', @number, '-', @number2)}"/>
          </xsl:when>
          <xsl:otherwise>
            <fo:inline rx:key="{$key}"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:if>
  </xsl:template>

  <!-- OMITTED: index-base, index-see, index-see-also, index-sort-as -->

  <!-- indextermref ====================================================== -->

  <xsl:template match="*[contains(@class,' topic/indextermref ')]"/>

  <!-- tm ================================================================ -->
  
  <xsl:attribute-set name="tm">
  </xsl:attribute-set>

  <xsl:template match="*[contains(@class,' topic/tm ')]">
    <fo:inline xsl:use-attribute-sets="tm">
      <xsl:call-template name="commonAttributes"/>
      <xsl:apply-templates/>
      <xsl:choose>
        <xsl:when test="@tmtype = 'reg'">&#x00AE;</xsl:when>
        <xsl:when test="@tmtype = 'service'">&#x2120;</xsl:when>
        <xsl:otherwise>&#x2122;</xsl:otherwise>
      </xsl:choose>
    </fo:inline>
  </xsl:template>

  <!-- data-about ======================================================== -->

  <xsl:template match="*[contains(@class,' topic/data-about ')]"/>

  <!-- data ============================================================== -->

  <xsl:template match="*[contains(@class,' topic/data ')]"/>

  <!-- foreign =========================================================== -->

  <xsl:template match="*[contains(@class,' topic/foreign ')]">
    <xsl:for-each select="./svg:svg|./mml:math">
      <fo:instream-foreign-object>
        <xsl:copy-of select="."/>
      </fo:instream-foreign-object>
    </xsl:for-each>
  </xsl:template>

  <!-- unknown =========================================================== -->

  <xsl:template match="*[contains(@class,' topic/unknown ')]"/>

</xsl:stylesheet>
