<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2009-2010 Pixware SARL. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind DITA Converter project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:ditac="http://www.xmlmind.com/ditac/schema/ditac"
                xmlns:URI="java:com.xmlmind.ditac.xslt.URI"
                xmlns:u="http://www.xmlmind.com/namespace/ditac"
                exclude-result-prefixes="xs u ditac URI"
                version="2.0">

  <xsl:attribute-set name="first-title-page">
    <xsl:attribute name="border-width">0.5pt</xsl:attribute>
    <xsl:attribute name="border-style">solid</xsl:attribute>
    <xsl:attribute name="border-color">#404040</xsl:attribute>
  </xsl:attribute-set>

  <xsl:template name="layoutMasterSet">
    <fo:layout-master-set>
      <!-- simple-page-masters =========================================== -->

      <!-- blank-page ========== -->

      <fo:simple-page-master master-name="blank-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"/>
        <fo:region-after extent="{$footer-height}"/>
      </fo:simple-page-master>

      <!-- title-page ========== -->

      <fo:simple-page-master master-name="first-title-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <xsl:choose>
          <xsl:when test="$foProcessor = 'XEP'">
            <!-- The border around the region-body is a XEP extension 
                 (Border and Padding on Regions).
                 In both XSL 1.0 and 1.1, the values of the padding and 
                 border-width traits must be "0".  -->
            <fo:region-body margin-top="{$body-top-margin}"
                            margin-bottom="{$body-bottom-margin}"
                            xsl:use-attribute-sets="first-title-page"/>
          </xsl:when>
          <xsl:otherwise>
            <fo:region-body margin-top="{$body-top-margin}"
                            margin-bottom="{$body-bottom-margin}"/>
          </xsl:otherwise>
        </xsl:choose>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="odd-title-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="even-title-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- toc-page ========== -->

      <fo:simple-page-master master-name="first-toc-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="odd-toc-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="even-toc-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- booklist-page ========== -->

      <fo:simple-page-master master-name="first-booklist-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="odd-booklist-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="even-booklist-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- frontmatter-page ========== -->

      <fo:simple-page-master master-name="first-frontmatter-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                          display-align="after"
                          region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="odd-frontmatter-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="even-frontmatter-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- body-page ========== -->

      <fo:simple-page-master master-name="first-body-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="odd-body-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                          display-align="after"
                          region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="even-body-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- backmatter-page ========== -->

      <fo:simple-page-master master-name="first-backmatter-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="odd-backmatter-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="even-backmatter-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- index-page ========== -->

      <fo:simple-page-master master-name="first-index-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"
                        column-count="{$index-column-count}" 
                        column-gap="{$index-column-gap}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="first-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="first-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="odd-index-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-inner-margin}"
                             margin-right="{$page-outer-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"
                        column-count="{$index-column-count}" 
                        column-gap="{$index-column-gap}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="odd-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="odd-page-footer"/>
      </fo:simple-page-master>

      <fo:simple-page-master master-name="even-index-page"
                             page-width="{$page-width}"
                             page-height="{$page-height}"
                             margin-top="{$page-top-margin}"
                             margin-bottom="{$page-bottom-margin}"
                             margin-left="{$page-outer-margin}"
                             margin-right="{$page-inner-margin}">
        <fo:region-body margin-top="{$body-top-margin}"
                        margin-bottom="{$body-bottom-margin}"
                        column-count="{$index-column-count}" 
                        column-gap="{$index-column-gap}"/>
        <fo:region-before extent="{$header-height}"
                          display-align="before"
                          region-name="even-page-header"/>
        <fo:region-after extent="{$footer-height}"
                         display-align="after"
                         region-name="even-page-footer"/>
      </fo:simple-page-master>

      <!-- page-sequence-masters ========================================= -->

      <!-- title ========== -->

      <fo:page-sequence-master master-name="title">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="first-title-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="odd-title-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="even-title-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- toc ========== -->

      <fo:page-sequence-master master-name="toc">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="first-toc-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="odd-toc-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="even-toc-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- booklist ========== -->

      <fo:page-sequence-master master-name="booklist">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="first-booklist-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="odd-booklist-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="even-booklist-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- frontmatter ========== -->

      <fo:page-sequence-master master-name="frontmatter">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="first-frontmatter-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="odd-frontmatter-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="even-frontmatter-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- body ========== -->

      <fo:page-sequence-master master-name="body">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="first-body-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="odd-body-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="even-body-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- backmatter ========== -->

      <fo:page-sequence-master master-name="backmatter">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="first-backmatter-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="odd-backmatter-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="even-backmatter-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>

      <!-- index ========== -->

      <fo:page-sequence-master master-name="index">
        <fo:repeatable-page-master-alternatives>
          <fo:conditional-page-master-reference 
            master-reference="blank-page"
            blank-or-not-blank="blank"/>
          <fo:conditional-page-master-reference 
            master-reference="first-index-page"
            page-position="first"/>
          <fo:conditional-page-master-reference
            master-reference="odd-index-page"
            odd-or-even="odd"/>
          <fo:conditional-page-master-reference 
            odd-or-even="even" 
            master-reference="even-index-page"/>
        </fo:repeatable-page-master-alternatives>
      </fo:page-sequence-master>
    </fo:layout-master-set>
  </xsl:template>

  <!-- configurePageSequence ============================================ -->

  <xsl:template name="configurePageSequence">
    <!-- The context node is the element (titlePage, toc, chapter, etc) which
         is the head of this page sequence. -->

    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="frontmatterNames" select="()"/>
    <xsl:param name="bodyNames" select="()"/>
    <xsl:param name="backmatterNames" select="()"/>
    <xsl:param name="topicInfo" select="element()"/>
    <xsl:param name="documentTitle" select="''"/>

    <!-- master-reference ========== -->

    <xsl:variable name="type" select="substring-before($sequenceName, '.')"/>

    <xsl:variable name="masterName">
      <xsl:choose>
        <xsl:when test="$type = 'titlePage'">
          <xsl:value-of select="'title'"/>
        </xsl:when>
        <xsl:when test="$type = 'toc'">
          <xsl:value-of select="'toc'"/>
        </xsl:when>
        <xsl:when test="$type = 'figureList' or 
                        $type = 'tableList' or 
                        $type = 'exampleList'">
          <xsl:value-of select="'booklist'"/>
        </xsl:when>
        <xsl:when test="$type = 'indexList'">
          <xsl:value-of select="'index'"/>
        </xsl:when>
        <xsl:when test="$type = 'part' or
                        $type = 'chapter' or
                        $type = 'appendices' or
                        $type = 'appendix' or
                        $type = 'section1' or
                        $type = 'section2' or
                        $type = 'section3' or
                        $type = 'section4' or
                        $type = 'section5' or
                        $type = 'section6' or
                        $type = 'section7' or
                        $type = 'section8' or
                        $type = 'section9'">
          <xsl:value-of select="'body'"/>
        </xsl:when>
        <xsl:otherwise>
          <!-- Frontmatter or backmatter other than toc, figureList, etc. That
               is, hand-written frontmatter or backmatter. Example:
               glossarylist. -->

          <xsl:value-of
            select="if (index-of($frontmatterNames, $sequenceName) ge 1) then 
                        'frontmatter' 
                    else 
                        'backmatter'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:attribute name="master-reference" select="$masterName"/>

    <!-- language (for hyphenation use) ========== -->

    <xsl:attribute name="language" select="u:documentLang()"/>

    <!-- hyphenation ========== -->

    <xsl:choose>
      <xsl:when test="$masterName = 'body' or 
                      $masterName = 'frontmatter' or 
                      $masterName = 'backmatter'">
        <xsl:attribute name="hyphenate" 
          select="if ($hyphenate = 'yes') then 'true' else 'false'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="hyphenate">false</xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>

    <!-- initial-page-number ========== -->

    <xsl:attribute name="initial-page-number">
      <xsl:choose>
        <xsl:when
          test="u:startsOnPage1($sequenceName, $frontmatterNames, $bodyNames, 
                                $backmatterNames)">1</xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when 
              test="$two-sided = 'yes' and 
                    u:startsOnOddPage($sequenceName, $frontmatterNames, 
                                      $bodyNames, 
                                      $backmatterNames)">auto-odd</xsl:when>
            <xsl:otherwise>auto</xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>

    <!-- force-page-count ========== -->

    <xsl:variable name="nextSequenceName" 
                  select="u:nextSequenceName($sequenceName, $frontmatterNames, 
                                             $bodyNames, $backmatterNames)"/>

    <xsl:attribute name="force-page-count" 
                   select="if ($two-sided = 'yes' and
                               $nextSequenceName != '' and
                               u:startsOnOddPage($nextSequenceName,
                                                 $frontmatterNames, $bodyNames,
                                                 $backmatterNames)) then 
                               'end-on-even'
                           else
                               'no-force'"/>

    <!-- format ========== -->

    <xsl:attribute name="format" 
                   select="if (index-of($bodyNames, $sequenceName) ge 1) then 
                               '1' 
                           else 
                               'i'"/>

    <!-- Footnote separator ========== -->

    <xsl:if test="$masterName = 'body' or 
                  $masterName = 'frontmatter' or 
                  $masterName = 'backmatter'">
      <xsl:call-template name="footnoteSeparator"/>
    </xsl:if>

    <!-- Headers and footers ========== -->

    <xsl:if test="$masterName != 'title'">
      <xsl:call-template name="headersAndFooters">
        <xsl:with-param name="sequenceName" select="$sequenceName"/>
        <xsl:with-param name="frontmatterNames" select="$frontmatterNames"/>
        <xsl:with-param name="bodyNames" select="$bodyNames"/>
        <xsl:with-param name="backmatterNames" select="$backmatterNames"/>
        <xsl:with-param name="topicInfo" select="$topicInfo"/>
        <xsl:with-param name="documentTitle" select="$documentTitle"/>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>
  
  <xsl:function name="u:startsOnPage1" as="xs:boolean">
    <xsl:param name="sequenceName" as="xs:string"/>
    <xsl:param name="frontmatterNames" as="xs:string*"/>
    <xsl:param name="bodyNames" as="xs:string*"/>
    <xsl:param name="backmatterNames" as="xs:string*"/>

    <xsl:variable name="frontmatterPos" 
                  select="index-of($frontmatterNames, $sequenceName)"/>

    <xsl:sequence 
      select="($frontmatterPos eq 1) or
              ($frontmatterPos gt 1 and 
               substring-before($frontmatterNames[$frontmatterPos - 1], 
                                '.') = 'titlePage') or
              (index-of($bodyNames, $sequenceName) eq 1) or
              (index-of($backmatterNames, $sequenceName) eq 1)"/>
  </xsl:function>

  <xsl:function name="u:startsOnOddPage" as="xs:boolean">
    <xsl:param name="sequenceName" as="xs:string"/>
    <xsl:param name="frontmatterNames" as="xs:string*"/>
    <xsl:param name="bodyNames" as="xs:string*"/>
    <xsl:param name="backmatterNames" as="xs:string*"/>

    <xsl:variable name="type" select="substring-before($sequenceName, '.')"/>

    <xsl:sequence select="$type = 'toc' or
                          $type = 'part' or
                          $type = 'chapter' or
                          $type = 'appendices' or
                          $type = 'appendix' or
                          $type = 'indexList' or
                          u:startsOnPage1($sequenceName, $frontmatterNames, 
                                          $bodyNames, $backmatterNames)"/>
  </xsl:function>

  <xsl:function name="u:nextSequenceName" as="xs:string">
    <xsl:param name="sequenceName" as="xs:string"/>
    <xsl:param name="frontmatterNames" as="xs:string*"/>
    <xsl:param name="bodyNames" as="xs:string*"/>
    <xsl:param name="backmatterNames" as="xs:string*"/>

    <xsl:variable name="list" 
                  select="$frontmatterNames, $bodyNames, $backmatterNames"/>
    <xsl:variable name="pos" select="index-of($list, $sequenceName)"/>
    <xsl:sequence 
      select="if ($pos ge 1 and $pos lt count($list)) then
                  $list[$pos + 1]
              else
                  ''"/>
  </xsl:function>

  <!-- headersAndFooters ================================================= -->

  <xsl:template name="headersAndFooters">
    <xsl:param name="sequenceName" select="''"/>
    <xsl:param name="frontmatterNames" select="()"/>
    <xsl:param name="bodyNames" select="()"/>
    <xsl:param name="backmatterNames" select="()"/>
    <xsl:param name="topicInfo" select="element()"/>
    <xsl:param name="documentTitle" select="''"/>

    <!-- About chapter-title:

         * One-sided document:

           First page: nothing.
           Even page: part, chapter, appendices or appendix title.
           Odd page: part, chapter, appendices or appendix title.

         * Two-sided document:

           First page: nothing.
           Even page: part, chapter, appendices or appendix title 
           Odd page: running ``chapter title'' which retrieves
                     part, chapter, appendices, appendix or 
                     part|chapter|appendix/section1 title.
    -->

    <!-- The chapter-title is displayed only for a part, chapter, appendices 
         or appendix page sequence. -->

    <xsl:variable name="type" select="substring-before($sequenceName, '.')"/>
    <xsl:variable name="hasChapterTitle" 
                  select="$type = 'part' or 
                          $type = 'chapter' or 
                          $type = 'appendices' or 
                          $type = 'appendix'"/>

    <xsl:variable name="staticChapterTitlePrefix" 
                  select="u:shortTitlePrefix(string($topicInfo/@number), 
                                             $topicInfo)" />

    <xsl:variable name="staticChapterTitle"
      select="if ($hasChapterTitle) 
              then 
                  if ($staticChapterTitlePrefix != '')
                  then concat($staticChapterTitlePrefix,
                              $title-prefix-separator1, 
                              $topicInfo/@title)
                  else string($topicInfo/@title)
              else ()"/>

    <xsl:variable name="runningChapterTitle">
      <fo:retrieve-marker retrieve-class-name="chapter-title"/>
    </xsl:variable>

    <xsl:variable name="firstPageChapterTitle" select="()"/>

    <xsl:variable name="evenPageChapterTitle" select="$staticChapterTitle" />

    <xsl:variable name="oddPageChapterTitle" 
                  select="if ($hasChapterTitle) 
                          then
                              if ($two-sided = 'yes')
                              then $runningChapterTitle
                              else $staticChapterTitle
                          else ()"/>

    <xsl:variable name="pageNumber">
      <fo:page-number/>
    </xsl:variable>

    <xsl:variable name="pageCount">
      <xsl:choose>
        <xsl:when test="index-of($frontmatterNames, $sequenceName) ge 1">
          <fo:page-number-citation ref-id="__EOFM"/>
        </xsl:when>
        <xsl:when test="index-of($bodyNames, $sequenceName) ge 1">
          <fo:page-number-citation ref-id="__EOBO"/>
        </xsl:when>
        <xsl:when test="index-of($backmatterNames, $sequenceName) ge 1">
          <fo:page-number-citation ref-id="__EOBM"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="hl1" 
      select="u:expandHeaderOrFooter($header-left-image, $header-left,
                                     $pageNumber, false(), $pageCount, 
                                     $firstPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="hc1" 
      select="u:expandHeaderOrFooter($header-center-image, $header-center,
                                     $pageNumber, false(), $pageCount, 
                                     $firstPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="hr1" 
      select="u:expandHeaderOrFooter($header-right-image, $header-right,
                                     $pageNumber, false(), $pageCount, 
                                     $firstPageChapterTitle, $documentTitle)"/>

    <xsl:if test="exists($hl1) or exists($hc1) or exists($hr1)">
      <fo:static-content flow-name="first-page-header">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$hl1"/>
          <xsl:with-param name="center" select="$hc1"/>
          <xsl:with-param name="right" select="$hr1"/>
          <xsl:with-param name="isFooter" select="false()"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

    <xsl:variable name="hlo" 
      select="u:expandHeaderOrFooter($header-left-image, $header-left,
                                     $pageNumber, false(), $pageCount, 
                                     $oddPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="hco" 
      select="u:expandHeaderOrFooter($header-center-image, $header-center,
                                     $pageNumber, false(), $pageCount, 
                                     $oddPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="hro" 
      select="u:expandHeaderOrFooter($header-right-image, $header-right,
                                     $pageNumber, false(), $pageCount, 
                                     $oddPageChapterTitle, $documentTitle)"/>

    <xsl:if test="exists($hlo) or exists($hco) or exists($hro)">
      <fo:static-content flow-name="odd-page-header">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$hlo"/>
          <xsl:with-param name="center" select="$hco"/>
          <xsl:with-param name="right" select="$hro"/>
          <xsl:with-param name="isFooter" select="false()"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

    <xsl:variable name="hle" 
      select="u:expandHeaderOrFooter($header-left-image, $header-left,
                                     $pageNumber, true(), $pageCount, 
                                     $evenPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="hce" 
      select="u:expandHeaderOrFooter($header-center-image, $header-center,
                                     $pageNumber, true(), $pageCount, 
                                     $evenPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="hre" 
      select="u:expandHeaderOrFooter($header-right-image, $header-right,
                                     $pageNumber, true(), $pageCount, 
                                     $evenPageChapterTitle, $documentTitle)"/>

    <xsl:if test="exists($hle) or exists($hce) or exists($hre)">
      <fo:static-content flow-name="even-page-header">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$hle"/>
          <xsl:with-param name="center" select="$hce"/>
          <xsl:with-param name="right" select="$hre"/>
          <xsl:with-param name="isFooter" select="false()"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

    <xsl:variable name="fl1" 
      select="u:expandHeaderOrFooter($footer-left-image, $footer-left,
                                     $pageNumber, false(), $pageCount, 
                                     $firstPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="fc1" 
      select="u:expandHeaderOrFooter($footer-center-image, $footer-center,
                                     $pageNumber, false(), $pageCount, 
                                     $firstPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="fr1" 
      select="u:expandHeaderOrFooter($footer-right-image, $footer-right,
                                     $pageNumber, false(), $pageCount, 
                                     $firstPageChapterTitle, $documentTitle)"/>

    <xsl:if test="exists($fl1) or exists($fc1) or exists($fr1)">
      <fo:static-content flow-name="first-page-footer">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$fl1"/>
          <xsl:with-param name="center" select="$fc1"/>
          <xsl:with-param name="right" select="$fr1"/>
          <xsl:with-param name="isFooter" select="true()"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

    <xsl:variable name="flo" 
      select="u:expandHeaderOrFooter($footer-left-image, $footer-left,
                                     $pageNumber, false(), $pageCount, 
                                     $oddPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="fco" 
      select="u:expandHeaderOrFooter($footer-center-image, $footer-center,
                                     $pageNumber, false(), $pageCount, 
                                     $oddPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="fro" 
      select="u:expandHeaderOrFooter($footer-right-image, $footer-right,
                                     $pageNumber, false(), $pageCount, 
                                     $oddPageChapterTitle, $documentTitle)"/>

    <xsl:if test="exists($flo) or exists($fco) or exists($fro)">
      <fo:static-content flow-name="odd-page-footer">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$flo"/>
          <xsl:with-param name="center" select="$fco"/>
          <xsl:with-param name="right" select="$fro"/>
          <xsl:with-param name="isFooter" select="true()"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

    <xsl:variable name="fle" 
      select="u:expandHeaderOrFooter($footer-left-image, $footer-left, 
                                     $pageNumber, true(), $pageCount, 
                                     $evenPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="fce" 
      select="u:expandHeaderOrFooter($footer-center-image, $footer-center, 
                                     $pageNumber, true(), $pageCount, 
                                     $evenPageChapterTitle, $documentTitle)"/>
    <xsl:variable name="fre" 
      select="u:expandHeaderOrFooter($footer-right-image, $footer-right, 
                                     $pageNumber, true(), $pageCount, 
                                     $evenPageChapterTitle, $documentTitle)"/>

    <xsl:if test="exists($fle) or exists($fce) or exists($fre)">
      <fo:static-content flow-name="even-page-footer">
        <xsl:call-template name="tabularHeader">
          <xsl:with-param name="left" select="$fle"/>
          <xsl:with-param name="center" select="$fce"/>
          <xsl:with-param name="right" select="$fre"/>
          <xsl:with-param name="isFooter" select="true()"/>
        </xsl:call-template>
      </fo:static-content>
    </xsl:if>

  </xsl:template>

  <xsl:function name="u:expandHeaderOrFooter" as="item()*">
    <xsl:param name="imageSpec" as="xs:string"/>
    <xsl:param name="textSpec" as="xs:string"/>

    <xsl:param name="pageNumber" as="item()*"/>
    <xsl:param name="isEven" as="xs:boolean"/>
    <xsl:param name="pageCount" as="item()*"/>
    <xsl:param name="chapterTitle" as="item()*"/>
    <xsl:param name="documentTitle" as="item()*"/>
    
    <xsl:variable name="oddPageNumber"
                  select="if ($two-sided = 'yes' and $isEven)
                          then ()
                          else $pageNumber"/>

    <xsl:variable name="evenPageNumber"
                  select="if ($two-sided = 'yes' and $isEven)
                          then $pageNumber
                          else ()"/>

    <xsl:choose>
      <xsl:when test="$imageSpec != ''">
        <xsl:variable name="imageURI" 
                      select="resolve-uri($imageSpec, URI:userDirectory())"/>

        <fo:external-graphic src="{concat('url(', $imageURI, ')')}"/>
      </xsl:when>

      <xsl:when test="$textSpec != ''">
        <xsl:variable name="textSpec2">
          <xsl:choose>
            <xsl:when test="contains($textSpec, '^') and 
                            not(contains($textSpec, '%'))">
              <!-- Support ^page-number^ rather than %page-number%. -->
              <xsl:value-of 
                select="translate(replace($textSpec, '\^\^', '&#xEEEF;'), 
                                  '^', '%')" />
            </xsl:when>

            <xsl:otherwise>
              <!-- &#xEEEE; belongs to Unicode Basic Multilingual Plane 
                   Private Use Area: U+E000 to U+F8FF -->
              <xsl:value-of select="replace($textSpec, '%%', '&#xEEEE;')" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:sequence 
          select="for $t in tokenize($textSpec2, '%')
                  return
                    if ($t = 'page-number') then $pageNumber
                    else 
                      if ($t = 'odd-page-number') then $oddPageNumber
                      else 
                        if ($t = 'even-page-number') then $evenPageNumber
                        else 
                          if ($t = 'page-count') then $pageCount
                          else 
                            if ($t = 'chapter-title') then $chapterTitle
                            else 
                              if ($t = 'document-title') then $documentTitle
                              else 
                                if ($t != '') 
                                then replace(replace($t, '&#xEEEF;', '^'), 
                                             '&#xEEEE;', '%')
                                else ()"/>
      </xsl:when>

      <xsl:otherwise>
        <xsl:sequence select="()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <!-- tabularHeader ===================================================== -->

  <xsl:attribute-set name="header-or-footer">
    <xsl:attribute name="hyphenate">false</xsl:attribute>
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
    <xsl:attribute name="font-size">70%</xsl:attribute>
    <xsl:attribute name="color">#404040</xsl:attribute>
    <xsl:attribute name="border-color">#404040</xsl:attribute>
    <xsl:attribute name="border-width">0.25pt</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header" use-attribute-sets="header-or-footer"/>

  <xsl:attribute-set name="footer" use-attribute-sets="header-or-footer"/>

  <xsl:attribute-set name="header-or-footer-left">
    <xsl:attribute name="text-align">left</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header-or-footer-center">
    <xsl:attribute name="text-align">center</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header-or-footer-right">
    <xsl:attribute name="text-align">right</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="header-left" 
                     use-attribute-sets="header-or-footer-left"/>
  <xsl:attribute-set name="header-center" 
                     use-attribute-sets="header-or-footer-center"/>
  <xsl:attribute-set name="header-right" 
                     use-attribute-sets="header-or-footer-right"/>

  <xsl:attribute-set name="footer-left" 
                     use-attribute-sets="header-or-footer-left"/>
  <xsl:attribute-set name="footer-center" 
                     use-attribute-sets="header-or-footer-center"/>
  <xsl:attribute-set name="footer-right" 
                     use-attribute-sets="header-or-footer-right"/>

  <xsl:param name="header-or-footer-left-width">proportional-column-width(2)</xsl:param>
  <xsl:param name="header-or-footer-center-width">proportional-column-width(6)</xsl:param>
  <xsl:param name="header-or-footer-right-width">proportional-column-width(2)</xsl:param>

  <xsl:param name="header-left-width" 
             select="$header-or-footer-left-width"/>
  <xsl:param name="header-center-width" 
             select="$header-or-footer-center-width"/>
  <xsl:param name="header-right-width" 
             select="$header-or-footer-right-width"/>

  <xsl:param name="footer-left-width" 
             select="$header-or-footer-left-width"/>
  <xsl:param name="footer-center-width" 
             select="$header-or-footer-center-width"/>
  <xsl:param name="footer-right-width" 
             select="$header-or-footer-right-width"/>

  <xsl:template name="tabularHeader">
    <xsl:param name="left" select="()"/>
    <xsl:param name="center" select="()"/>
    <xsl:param name="right" select="()"/>
    <xsl:param name="isFooter" select="false()"/>

    <xsl:choose>
      <xsl:when test="$isFooter">
        <fo:block xsl:use-attribute-sets="footer">
          <xsl:if test="$footer-separator = 'yes'">
            <xsl:attribute name="border-top-style">solid</xsl:attribute>
            <xsl:attribute name="padding-top">0.125em</xsl:attribute>
          </xsl:if>

          <fo:table table-layout="fixed" width="100%">
            <fo:table-column column-number="1" 
                             column-width="{$footer-left-width}"/>
            <fo:table-column column-number="2" 
                             column-width="{$footer-center-width}"/>
            <fo:table-column column-number="3" 
                             column-width="{$footer-right-width}"/>
            <fo:table-body>
              <fo:table-row>
                <fo:table-cell start-indent="0" display-align="before"
                               xsl:use-attribute-sets="footer-left">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($left)">
                        <xsl:copy-of select="$left"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="before"
                               xsl:use-attribute-sets="footer-center">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($center)">
                        <xsl:copy-of select="$center"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="before"
                               xsl:use-attribute-sets="footer-right">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($right)">
                        <xsl:copy-of select="$right"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
              </fo:table-row>
            </fo:table-body>
          </fo:table>
        </fo:block>
      </xsl:when>

      <xsl:otherwise>
        <fo:block xsl:use-attribute-sets="header">
          <xsl:if test="$header-separator = 'yes'">
            <xsl:attribute name="padding-bottom">0.125em</xsl:attribute>
            <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
          </xsl:if>

          <fo:table table-layout="fixed" width="100%">
            <fo:table-column column-number="1" 
                             column-width="{$header-left-width}"/>
            <fo:table-column column-number="2" 
                             column-width="{$header-center-width}"/>
            <fo:table-column column-number="3" 
                             column-width="{$header-right-width}"/>
            <fo:table-body>
              <fo:table-row>
                <fo:table-cell start-indent="0" display-align="after"
                               xsl:use-attribute-sets="header-left">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($left)">
                        <xsl:copy-of select="$left"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="after"
                               xsl:use-attribute-sets="header-center">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($center)">
                        <xsl:copy-of select="$center"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
                <fo:table-cell start-indent="0" display-align="after"
                               xsl:use-attribute-sets="header-right">
                  <fo:block>
                    <xsl:choose>
                      <xsl:when test="exists($right)">
                        <xsl:copy-of select="$right"/>
                      </xsl:when>
                      <xsl:otherwise>&#xA0;</xsl:otherwise>
                    </xsl:choose>
                  </fo:block>
                </fo:table-cell>
              </fo:table-row>
            </fo:table-body>
          </fo:table>
        </fo:block>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- footnoteSeparator ================================================= -->

  <xsl:attribute-set name="footnote-separator">
    <xsl:attribute name="leader-pattern">rule</xsl:attribute>
    <xsl:attribute name="leader-length">30%</xsl:attribute>
    <xsl:attribute name="rule-style">solid</xsl:attribute>
    <xsl:attribute name="color">#404040</xsl:attribute>
    <xsl:attribute name="rule-thickness">0.25pt</xsl:attribute>
  </xsl:attribute-set>

  <xsl:template name="footnoteSeparator">
    <fo:static-content flow-name="xsl-footnote-separator">
      <fo:block>
        <fo:leader xsl:use-attribute-sets="footnote-separator"/>
      </fo:block>
    </fo:static-content>
  </xsl:template>

</xsl:stylesheet>
