<?xml version="1.0" encoding="UTF-8"?>
<!--
| Copyright (c) 2009-2011 Pixware SARL. All rights reserved.
|
| Author: Hussein Shafie
|
| This file is part of the XMLmind DITA Converter project.
| For conditions of distribution and use, see the accompanying LEGAL.txt file.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:u="http://www.xmlmind.com/namespace/ditac"
                xmlns:ditac="http://www.xmlmind.com/ditac/schema/ditac"
                exclude-result-prefixes="xs u ditac"
                version="2.0">

  <xsl:template match="ditac:toc">
    <div class="toc-container" id="__TOC">
      <h1 class="toc-title">
        <xsl:call-template name="localize">
          <xsl:with-param name="message" select="'toc'"/>
        </xsl:call-template>
      </h1>
      <div class="toc">
        <xsl:apply-templates select="$ditacLists/ditac:toc/ditac:tocEntry"/>
      </div>
    </div>
  </xsl:template>

  <xsl:template match="ditac:tocEntry">
    <xsl:variable name="isDiv" 
      select="((@role = 'part' or @role = 'appendices') and 
               u:indexOfNode(parent::ditac:toc/ditac:tocEntry, .) ge 2) or
              (@role = 'appendix' and 
               empty(parent::ditac:tocEntry[@role = 'appendices']) and 
               empty(preceding::ditac:tocEntry[@role = 'appendix']))" />

    <div class="{if ($isDiv) 
                 then 'toc-entry-division'
                 else 'toc-entry-container'}">
      <p class="{concat('toc-', string(@role), '-entry')}">
        <span class="toc-entry-number">
          <xsl:variable name="num"
                        select="u:shortTitlePrefix(string(@number), .)"/>
          <xsl:choose>
            <xsl:when test="$num != ''">
              <xsl:choose>
                <xsl:when test="@role = 'part' or
                                @role = 'chapter' or
                                @role = 'appendix'">
                  <xsl:value-of 
                    select="concat($num, $title-prefix-separator1)"/>
                </xsl:when>
                <xsl:otherwise>
                  <!-- Discard leading 'Section '. -->
                  <xsl:value-of 
                    select="concat(substring-after($num, '&#xA0;'), 
                                   $title-prefix-separator1)"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:otherwise>&#xA0;</xsl:otherwise>
          </xsl:choose>
        </span>

        <a href="{u:joinLinkTarget(@file, @id)}" 
           class="toc-entry-title"><xsl:value-of select="@title"/></a>
      </p>

      <xsl:variable name="nested" select="child::ditac:tocEntry"/>
      <xsl:if test="exists($nested)">
        <div class="toc-nested-entries">
          <xsl:apply-templates select="$nested"/>
        </div>
      </xsl:if>
    </div>
  </xsl:template>

</xsl:stylesheet>
