<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:doc="http://nwalsh.com/xsl/documentation/1.0"
  xmlns:ng="http://docbook.org/docbook-ng"
  xmlns:db="http://docbook.org/ns/docbook"
  xmlns:exsl="http://exslt.org/common"
  xmlns:set="http://exslt.org/sets"
  xmlns:uu="java:com.xmlmind.util.URLUtil"
  xmlns:uuid="java:java.util.UUID"
  xmlns:opf="http://www.idpf.org/2007/opf"
  xmlns:ncx="http://www.daisy.org/z3986/2005/ncx/"
  xmlns:dc="http://purl.org/dc/elements/1.1/"
  xmlns:dcterms="http://purl.org/dc/terms/"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  version="1.1"
  exclude-result-prefixes="doc ng db exsl set uu uuid">

<!-- =========================================================================
  Requires removing the doctype-public and doctype-system attributes 
  in the xsl:output element found in ../xhtml-1_1/docbook.xsl.
  Given the way write.chunk has been written, this will not prevent
  the generated XHTML 1.1 files from having the proper DOCTYPE.
========================================================================== -->

<xsl:import href="../xhtml-1_1/chunk.xsl" />

<xsl:param name="epub.identifier" select="''" />

<xsl:variable name="epub.uuid">
  <xsl:if test="function-available('uuid:toString') and 
                function-available('uuid:randomUUID')">
    <xsl:value-of select="concat('urn:uuid:',uuid:toString(uuid:randomUUID()))" />
  </xsl:if>
</xsl:variable>

<xsl:param name="epub.cover.image" select="''" />

<xsl:variable name="epub.coverImageName">
  <xsl:if test="$epub.cover.image != ''">
    <xsl:variable name="ext">
      <xsl:call-template name="filename-extension">
        <xsl:with-param name="filename" select="$epub.cover.image"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test="$ext != ''">
      <xsl:value-of select="concat('cover.',$ext)"/>
    </xsl:if>
  </xsl:if>
</xsl:variable>

<xsl:variable name="playOrder0">
  <xsl:choose>
    <xsl:when test="$rootid != ''">
      <xsl:apply-templates select="key('id',$rootid)" mode="playOrder0"/>
    </xsl:when>
    <xsl:otherwise>1</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:template match="*" mode="playOrder0">
  <xsl:number level="any"
    count="set|book|part|reference|preface|
           chapter|appendix|article|
           section|sect1|sect2|sect3|sect4|sect5|
           colophon|refentry|glossary|bibliography"/>
</xsl:template>

<!-- ===================================================================== -->

<xsl:template match="/">
  <xsl:if test="not(function-available('exsl:node-set')) or 
                not(function-available('set:distinct'))">
    <xsl:message terminate="yes">
      <xsl:text>Extension functions 'exsl:node-set' and</xsl:text>
      <xsl:text> 'set:distinct' are not implemented by </xsl:text>
      <xsl:value-of select="system-property('xsl:vendor')"/>
      <xsl:text>.</xsl:text>
    </xsl:message>
  </xsl:if>

  <xsl:choose>
    <xsl:when test="$rootid != ''">
      <xsl:choose>
        <xsl:when test="count(key('id',$rootid)) = 0">
          <xsl:message terminate="yes">
            <xsl:text>ID '</xsl:text>
            <xsl:value-of select="$rootid"/>
            <xsl:text>' not found in document.</xsl:text>
          </xsl:message>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>Formatting from 
          <xsl:value-of select="$rootid"/></xsl:message>
          <xsl:apply-templates select="key('id',$rootid)" mode="process.root"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="/" mode="process.root"/>
    </xsl:otherwise>
  </xsl:choose>

  <!-- This for-each is a hook for building profiling stylesheets -->
  <xsl:for-each select="/">    
    <xsl:if test="$epub.coverImageName != ''">
      <xsl:call-template name="cover"/>
    </xsl:if>
    <xsl:call-template name="opf"/>
    <xsl:call-template name="toc"/>
  </xsl:for-each>
</xsl:template>

<xsl:template name="header.navigation">
</xsl:template>

<xsl:template name="footer.navigation">
</xsl:template>

<!-- ===================================================================== -->

<xsl:template name="toc">
  <xsl:call-template name="write.chunk.with.doctype">
    <xsl:with-param name="filename" select="concat($base.dir,'toc.ncx')"/>
    <xsl:with-param name="method" select="'xml'"/>
    <xsl:with-param name="indent" select="'yes'"/>
    <xsl:with-param name="doctype-public" 
                    select="'-//NISO//DTD ncx 2005-1//EN'"/>
    <xsl:with-param name="doctype-system" 
                    select="'http://www.daisy.org/z3986/2005/ncx-2005-1.dtd'"/>
    <xsl:with-param name="content">
      <xsl:call-template name="toc.content"/>
    </xsl:with-param>
    <xsl:with-param name="quiet" select="$chunk.quietly"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="toc.content">
  <xsl:variable name="title">
    <xsl:call-template name="root.title" />
  </xsl:variable>

  <xsl:variable name="navMap">
    <ncx:navMap>
      <xsl:choose>
        <xsl:when test="$rootid != ''">
          <xsl:apply-templates select="key('id',$rootid)" mode="toc.ncx"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="." mode="toc.ncx"/>
        </xsl:otherwise>
      </xsl:choose>
    </ncx:navMap>
  </xsl:variable>

  <xsl:variable name="depth">
    <xsl:call-template name="toc.depth">
      <xsl:with-param name="ns" select="exsl:node-set($navMap)" />
    </xsl:call-template>
  </xsl:variable>

  <ncx:ncx version="2005-1">
    <ncx:head>
      <ncx:meta name="dtb:uid">
        <xsl:attribute name="content">
          <xsl:choose>
            <xsl:when test="$rootid != ''">
              <xsl:call-template name="epub.identifier">
                <xsl:with-param name="root" select="key('id',$rootid)" />
              </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="epub.identifier">
                <xsl:with-param name="root" select="." />
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
      </ncx:meta>
      <ncx:meta name="dtb:depth" content="{$depth}"/>
      <ncx:meta name="dtb:totalPageCount" content="0"/>
      <ncx:meta name="dtb:maxPageNumber" content="0"/>
    </ncx:head>

    <ncx:docTitle>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:docTitle>

    <xsl:copy-of select="$navMap"/>
  </ncx:ncx>
</xsl:template>

<xsl:template name="root.title">
  <xsl:choose>
    <xsl:when test="$rootid != ''">
      <xsl:apply-templates select="key('id',$rootid)" mode="title.markup"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="." mode="title.markup"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="epub.identifier">
  <xsl:param name="root" select="." />

  <xsl:choose>
    <xsl:when test="$epub.identifier != ''">
      <xsl:value-of select="$epub.identifier" />
    </xsl:when>
    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test="not($root/..)">
          <!-- $root is the document -->

          <xsl:choose>
            <xsl:when test="$root/*/*/isbn">
              <xsl:value-of select="concat('isbn:', $root/*/*/isbn)" />
            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="epub.uuid" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:otherwise>
          <!-- $root is the topmost element -->

          <xsl:choose>
            <xsl:when test="$root/*/isbn">
              <xsl:value-of select="concat('isbn:', $root/*/isbn)" />
            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="epub.uuid" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="epub.uuid">
  <xsl:choose>
    <xsl:when test="$epub.uuid != ''">
      <xsl:value-of select="$epub.uuid"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:message terminate="yes">
        <xsl:text>Don't know how to generate a unique identifier</xsl:text>
        <xsl:text> for this document. Please specify a value</xsl:text> 
        <xsl:text> for parameter 'epub.identifier'.</xsl:text>
      </xsl:message>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="toc.depth">
  <xsl:param name="ns" select="''" />

  <xsl:choose>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">20</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">19</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">18</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">17</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">16</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">15</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">14</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">13</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">12</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">11</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">10</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">9</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">8</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">7</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">6</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">5</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint/ncx:navPoint">4</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint/ncx:navPoint">3</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint/ncx:navPoint">2</xsl:when>
    <xsl:when 
      test="$ns/ncx:navMap/ncx:navPoint">1</xsl:when>
    <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="set" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="book" mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="book" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="part|reference|preface|
                                 chapter|appendix|article|
                                 colophon|glossary|bibliography" 
                         mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="part|reference|preface|
                     chapter|appendix|article" 
                     mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="article|preface|chapter|
                                 appendix|refentry|
                                 section|sect1|
                                 glossary|bibliography" 
                         mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="section" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="section" mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="sect1" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="sect2" mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="sect2" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="sect3" mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="sect3" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="sect4" mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="sect4" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>

    <xsl:apply-templates select="sect5" mode="toc.ncx"/>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="sect5|colophon|refentry" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:apply-templates select="." mode="title.markup"/>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="glossary" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:call-template name="gentext">
      <xsl:with-param name="key" select="'Glossary'"/>
    </xsl:call-template>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>
  </ncx:navPoint>
</xsl:template>

<xsl:template match="bibliography" mode="toc.ncx">
  <xsl:variable name="id">
    <xsl:call-template name="object.id">
      <xsl:with-param name="object" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="title">
    <xsl:call-template name="gentext">
      <xsl:with-param name="key" select="'Bibliography'"/>
    </xsl:call-template>
  </xsl:variable>

  <ncx:navPoint id="{$id}">
    <xsl:attribute name="playOrder">
      <xsl:call-template name="play.order"/>
    </xsl:attribute>

    <ncx:navLabel>
      <ncx:text><xsl:value-of select="normalize-space($title)"/></ncx:text>
    </ncx:navLabel>

    <ncx:content>
      <xsl:attribute name="src">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
    </ncx:content>
  </ncx:navPoint>
</xsl:template>

<xsl:template name="play.order">
  <xsl:variable name="po">
    <xsl:number level="any"
      count="set|book|part|reference|preface|
             chapter|appendix|article|
             section|sect1|sect2|sect3|sect4|sect5|
             colophon|refentry|glossary|bibliography"/>
  </xsl:variable>

  <xsl:value-of select="$po - $playOrder0 + 1"/>
</xsl:template>

<!-- ===================================================================== -->

<xsl:template name="cover">
  <xsl:call-template name="write.chunk">
    <xsl:with-param name="filename" select="concat($base.dir,'cover.xml')"/>
    <xsl:with-param name="content">
      <xsl:call-template name="cover.content"/>
    </xsl:with-param>
    <xsl:with-param name="quiet" select="$chunk.quietly"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="cover.content">
  <xsl:variable name="title">
    <xsl:call-template name="root.title" />
  </xsl:variable>

  <xsl:variable name="dst" select="concat($base.dir,$epub.coverImageName)" />

  <html xmlns="http://www.w3.org/1999/xhtml">
    <head>
      <title>Cover</title>
      <style type="text/css">img { max-width: 100%; }</style>
    </head>
    <body>
      <!-- A simple way to ensure that URLUtil:copyFile is invoked. -->
      <xsl:comment>
        <xsl:value-of select="concat('&quot;', $epub.cover.image, '&quot;: ', 
                                     uu:copyFile($epub.cover.image, $dst), 
                                     'bytes')" />
      </xsl:comment>

      <div id="cover-image">
        <img src="{$epub.coverImageName}" alt="{normalize-space($title)}" />
      </div>
    </body>
  </html>
</xsl:template>

<!-- ===================================================================== -->

<xsl:template name="opf">
  <xsl:call-template name="write.chunk.with.doctype">
    <xsl:with-param name="filename" select="concat($base.dir,'content.opf')"/>
    <xsl:with-param name="method" select="'xml'"/>
    <xsl:with-param name="indent" select="'yes'"/>
    <xsl:with-param name="doctype-public" select="''"/>
    <xsl:with-param name="doctype-system" select="''"/>
    <xsl:with-param name="content">
      <xsl:call-template name="opf.content"/>
    </xsl:with-param>
    <xsl:with-param name="quiet" select="$chunk.quietly"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="opf.content">
  <opf:package unique-identifier="__IDENTIFIER__" version="2.0">
    <xsl:call-template name="opf.metadata"/>

    <xsl:variable name="manifest">
      <xsl:call-template name="opf.manifest"/>
    </xsl:variable>
    <xsl:copy-of select="$manifest"/>

    <xsl:call-template name="opf.spine">
      <xsl:with-param name="ns" select="exsl:node-set($manifest)" />
    </xsl:call-template>

    <xsl:call-template name="opf.guide">
      <xsl:with-param name="ns" select="exsl:node-set($manifest)" />
    </xsl:call-template>
  </opf:package>
</xsl:template>

<!-- ========== metadata ========== -->

<xsl:template name="opf.metadata">
  <xsl:choose>
    <xsl:when test="$rootid != ''">
      <xsl:call-template name="opf.metadata.section">
        <xsl:with-param name="root" select="key('id',$rootid)" />
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="opf.metadata.section">
        <xsl:with-param name="root" select="." />
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="opf.metadata.section">
  <xsl:param name="root" select="." />

  <opf:metadata>
    <dc:title>
      <xsl:apply-templates select="$root" mode="title.markup"/>
    </dc:title>

    <dc:identifier id="__IDENTIFIER__">
      <xsl:call-template name="epub.identifier">
        <xsl:with-param name="root" select="$root" />
      </xsl:call-template>
    </dc:identifier>

    <xsl:choose>
      <xsl:when test="not($root/..)">
        <!-- $root is the document -->

        <dc:language xsi:type="dcterms:RFC3066">
          <xsl:choose>
            <xsl:when test="$root/*/@lang">
              <xsl:value-of select="$root/*/@lang" />
            </xsl:when>
            <xsl:otherwise>en</xsl:otherwise>
          </xsl:choose>
        </dc:language>

        <xsl:for-each select="$root/*/*/author|$root/*/*/corpauthor">
          <xsl:variable name="author">
            <!-- This utility found in common generates formatted markup -->
            <xsl:call-template name="person.name">
              <xsl:with-param name="node" select="." />
            </xsl:call-template>
          </xsl:variable>

          <dc:creator opf:role="aut">
            <xsl:value-of select="$author" />
          </dc:creator>
        </xsl:for-each>

        <xsl:for-each select="$root/*/*/publishername|
                              $root/*/*/publisher/publishername">
          <dc:publisher><xsl:value-of select="." /></dc:publisher>
        </xsl:for-each>
      </xsl:when>

      <xsl:otherwise>
        <!-- $root is the topmost element -->

        <dc:language xsi:type="dcterms:RFC3066">
          <xsl:choose>
            <xsl:when test="$root/@lang">
              <xsl:value-of select="$root/@lang" />
            </xsl:when>
            <xsl:otherwise>en</xsl:otherwise>
          </xsl:choose>
        </dc:language>

        <xsl:for-each select="$root/*/author|$root/*/corpauthor">
          <xsl:variable name="author">
            <xsl:call-template name="person.name">
              <xsl:with-param name="node" select="." />
            </xsl:call-template>
          </xsl:variable>

          <dc:creator opf:role="aut">
            <xsl:value-of select="$author" />
          </dc:creator>
        </xsl:for-each>

        <xsl:for-each select="$root/*/publishername|
                              $root/*/publisher/publishername">
          <dc:publisher><xsl:value-of select="." /></dc:publisher>
        </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>

    <xsl:if test="$epub.coverImageName != ''">
      <opf:meta name="cover" content="cover-image"/>
    </xsl:if>
  </opf:metadata>
</xsl:template>

<!-- ========== manifest ========== -->

<xsl:template name="opf.manifest">
  <opf:manifest>
    <xsl:if test="$epub.coverImageName != ''">
      <opf:item id="cover" href="cover.xml" 
                media-type="application/xhtml+xml"/>

      <opf:item id="cover-image" href="{$epub.coverImageName}">
        <xsl:attribute name="media-type">
          <xsl:call-template name="mime.type.for.suffix">
            <xsl:with-param name="suffix" select="$epub.coverImageName"/>
          </xsl:call-template>
        </xsl:attribute>
      </opf:item>
    </xsl:if>

    <opf:item id="__NCX__" href="toc.ncx" 
              media-type="application/x-dtbncx+xml" />

    <xsl:if test="$html.stylesheet != ''">
      <opf:item id="__CSS__" href="{$html.stylesheet}" 
                media-type="text/css" />
    </xsl:if>

    <xsl:choose>
      <xsl:when test="$rootid != ''">
        <xsl:apply-templates select="key('id',$rootid)" mode="opf.manifest" />

        <xsl:call-template name="list.all.images">
          <xsl:with-param name="root" select="key('id',$rootid)" />
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="." mode="opf.manifest" />

        <xsl:call-template name="list.all.images">
          <xsl:with-param name="root" select="." />
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </opf:manifest>
</xsl:template>

<xsl:template match="set" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <!-- This skips URIs ending with a fragment and thus,
       keeps just the actual files. -->

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="book" mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="book" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="part|reference|preface|
                                 chapter|appendix|article|
                                 colophon|glossary|bibliography" 
                         mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="part|reference|preface|
                     chapter|appendix|article" 
              mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="article|preface|chapter|
                                 appendix|refentry|
                                 section|sect1|
                                 glossary|bibliography" 
                         mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="section" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="section" mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="sect1" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="sect2" mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="sect2" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="sect3" mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="sect3" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="sect4" mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="sect4" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />

    <xsl:apply-templates select="sect5" mode="opf.manifest"/>
  </xsl:if>
</xsl:template>

<xsl:template match="sect5|colophon|refentry" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />
  </xsl:if>
</xsl:template>

<xsl:template match="glossary" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />
  </xsl:if>
</xsl:template>

<xsl:template match="bibliography" mode="opf.manifest">
  <xsl:variable name="uri">
    <xsl:call-template name="href.target.uri"/>
  </xsl:variable>

  <xsl:if test="substring($uri, 1+string-length($uri)-5) = '.html'">
    <xsl:variable name="id">
      <xsl:call-template name="object.id">
        <xsl:with-param name="object" select="."/>
      </xsl:call-template>
    </xsl:variable>
    <opf:item id="{$id}" href="{$uri}" media-type="application/xhtml+xml" />
  </xsl:if>
</xsl:template>

<xsl:template name="list.all.images">
  <xsl:param name="root" select="." />

  <xsl:variable name="all" 
    select="set:distinct($root//imagedata/@fileref|$root//graphic/@fileref|$root//inlinegraphic/@fileref)" />

  <xsl:call-template name="list.images">
    <xsl:with-param name="all" select="$all" />
    <xsl:with-param name="suffix" select="'.gif'" />
    <xsl:with-param name="mime" select="'image/gif'" />
  </xsl:call-template>

  <xsl:call-template name="list.images">
    <xsl:with-param name="all" select="$all" />
    <xsl:with-param name="suffix" select="'.png'" />
    <xsl:with-param name="mime" select="'image/png'" />
  </xsl:call-template>

  <xsl:call-template name="list.images">
    <xsl:with-param name="all" select="$all" />
    <xsl:with-param name="suffix" select="'.jpeg'" />
    <xsl:with-param name="mime" select="'image/jpeg'" />
  </xsl:call-template>

  <xsl:call-template name="list.images">
    <xsl:with-param name="all" select="$all" />
    <xsl:with-param name="suffix" select="'.jpg'" />
    <xsl:with-param name="mime" select="'image/jpeg'" />
  </xsl:call-template>

  <xsl:call-template name="list.images">
    <xsl:with-param name="all" select="$all" />
    <xsl:with-param name="suffix" select="'.svg'" />
    <xsl:with-param name="mime" select="'image/svg+xml'" />
  </xsl:call-template>

  <!-- Add XSL style sheet resources to the list. -->

  <xsl:choose>
    <xsl:when test="not($root/..)">
      <!-- $root is the document -->

      <xsl:if test="$root/*/@status = 'draft' and 
                    $draft.watermark.image != '' and
                    not(contains($draft.watermark.image, '://'))">
        <xsl:variable name="mime">
          <xsl:call-template name="mime.type.for.suffix">
            <xsl:with-param name="suffix" select="$draft.watermark.image"/>
          </xsl:call-template>
        </xsl:variable>

        <opf:item id="__IMAGE__draft" href="{$draft.watermark.image}" 
                  media-type="{$mime}" />
      </xsl:if>
    </xsl:when>

    <xsl:otherwise>
      <!-- $root is the topmost element -->

      <xsl:if test="$root/@status = 'draft' and 
                    $draft.watermark.image != '' and
                    not(contains($draft.watermark.image, '://'))">
        <xsl:variable name="mime">
          <xsl:call-template name="mime.type.for.suffix">
            <xsl:with-param name="suffix" select="$draft.watermark.image"/>
          </xsl:call-template>
        </xsl:variable>

        <opf:item id="__IMAGE__draft" href="{$draft.watermark.image}" 
                  media-type="{$mime}" />
      </xsl:if>
    </xsl:otherwise>
  </xsl:choose>

  <xsl:if test="$callout.graphics = 1 and $root//callout">
    <xsl:call-template name="list.callouts">
      <xsl:with-param name="i" select="$callout.graphics.number.limit" />
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<xsl:template name="list.images">
  <xsl:param name="all" select="''" />
  <xsl:param name="suffix" select="'.gif'" />
  <xsl:param name="mime" select="'image/gif'" />

  <xsl:for-each select="$all">
    <xsl:if
      test="substring(., 1+string-length(.)-string-length($suffix)) = $suffix">
      <opf:item id="{generate-id(.)}" href="{.}" media-type="{$mime}" />
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template name="list.callouts">
  <xsl:param name="i" select="0" />

  <xsl:if test="$i &gt; 0">
    <xsl:variable name="mime">
      <xsl:call-template name="mime.type.for.suffix">
        <xsl:with-param name="suffix" select="$callout.graphics.extension"/>
      </xsl:call-template>
    </xsl:variable>

    <opf:item id="{concat('__IMAGE__callout', $i)}" 
      href="{concat($callout.graphics.path, $i, $callout.graphics.extension)}" 
      media-type="{$mime}" />

    <xsl:call-template name="list.callouts">
      <xsl:with-param name="i" select="$i - 1" />
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<xsl:template name="mime.type.for.suffix">
  <xsl:param name="suffix" select="'.png'" />

  <xsl:choose>
    <xsl:when test="substring($suffix, 1+string-length($suffix)-4) = '.png'"
      >image/png</xsl:when>
    <xsl:when test="substring($suffix, 1+string-length($suffix)-4) = '.gif'"
      >image/gif</xsl:when>
    <xsl:when test="substring($suffix, 1+string-length($suffix)-4) = '.jpg'"
      >image/jpeg</xsl:when>
    <xsl:when test="substring($suffix, 1+string-length($suffix)-5) = '.jpeg'"
      >image/jpeg</xsl:when>
    <xsl:when test="substring($suffix, 1+string-length($suffix)-4) = '.svg'"
      >image/svg+xml</xsl:when>
  </xsl:choose>
</xsl:template>

<!-- ========== spine ========== -->

<xsl:template name="opf.spine">
  <xsl:param name="ns" select="''" />

  <opf:spine toc="__NCX__">
    <xsl:for-each 
      select="$ns/opf:manifest/opf:item[@media-type='application/xhtml+xml']">
      <opf:itemref idref="{@id}" />
    </xsl:for-each>
  </opf:spine>
</xsl:template>

<!-- ========== guide ========== -->

<xsl:template name="opf.guide">
  <xsl:param name="ns" select="''" />

  <xsl:variable name="title">
    <xsl:call-template name="root.title" />
  </xsl:variable>

  <xsl:variable name="firstItem" 
    select="($ns/opf:manifest/opf:item[@media-type = 'application/xhtml+xml' 
                                       and @href != 'cover.xml'])[1]"/>

  <!-- Minimal guide. Used just to declare our cover page.
       The guide section seems: 
       [1] deprecated by the standard,
       [2] ignored by all the EPUB readers we have tested. -->

  <opf:guide>
    <xsl:if test="$epub.coverImageName != ''">
      <opf:reference type="cover" title="Cover" href="cover.xml"/>
    </xsl:if>

    <opf:reference type="text" title="{normalize-space($title)}" 
                   href="{$firstItem/@href}"/>
  </opf:guide>
</xsl:template>

</xsl:stylesheet>
