<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:html="http://www.w3.org/1999/xhtml">

  <xsl:import href="xhtml2fo.xsl"/>

  <xsl:output method="xml"
              version="1.0"
              encoding="UTF-8"
              indent="no"/>

  <xsl:preserve-space elements="html:pre html:style html:script"/>

  <!--======================================================================
      Parameters
  =======================================================================-->

  <!-- page size -->
  <xsl:param name="paper-type">USletter</xsl:param>
  <xsl:param name="page-orientation">portrait</xsl:param>

  <xsl:param name="page-width">
    <xsl:choose>
      <xsl:when test="$page-orientation = 'portrait'">
        <xsl:value-of select="$page-width-portrait"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$page-height-portrait"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:param>

  <xsl:param name="page-height">
    <xsl:choose>
      <xsl:when test="$page-orientation = 'portrait'">
        <xsl:value-of select="$page-height-portrait"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$page-width-portrait"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:param>

  <xsl:param name="page-width-portrait">
    <xsl:choose>
      <xsl:when test="$paper-type = 'USletter'">8.5in</xsl:when>
      <xsl:when test="$paper-type = '4A0'">1682mm</xsl:when>
      <xsl:when test="$paper-type = '2A0'">1189mm</xsl:when>
      <xsl:when test="$paper-type = 'A0'">841mm</xsl:when>
      <xsl:when test="$paper-type = 'A1'">594mm</xsl:when>
      <xsl:when test="$paper-type = 'A2'">420mm</xsl:when>
      <xsl:when test="$paper-type = 'A3'">297mm</xsl:when>
      <xsl:when test="$paper-type = 'A4'">210mm</xsl:when>
      <xsl:when test="$paper-type = 'A5'">148mm</xsl:when>
      <xsl:when test="$paper-type = 'A6'">105mm</xsl:when>
      <xsl:when test="$paper-type = 'A7'">74mm</xsl:when>
      <xsl:when test="$paper-type = 'A8'">52mm</xsl:when>
      <xsl:when test="$paper-type = 'A9'">37mm</xsl:when>
      <xsl:when test="$paper-type = 'A10'">26mm</xsl:when>
      <xsl:when test="$paper-type = 'B0'">1000mm</xsl:when>
      <xsl:when test="$paper-type = 'B1'">707mm</xsl:when>
      <xsl:when test="$paper-type = 'B2'">500mm</xsl:when>
      <xsl:when test="$paper-type = 'B3'">353mm</xsl:when>
      <xsl:when test="$paper-type = 'B4'">250mm</xsl:when>
      <xsl:when test="$paper-type = 'B5'">176mm</xsl:when>
      <xsl:when test="$paper-type = 'B6'">125mm</xsl:when>
      <xsl:when test="$paper-type = 'B7'">88mm</xsl:when>
      <xsl:when test="$paper-type = 'B8'">62mm</xsl:when>
      <xsl:when test="$paper-type = 'B9'">44mm</xsl:when>
      <xsl:when test="$paper-type = 'B10'">31mm</xsl:when>
      <xsl:when test="$paper-type = 'C0'">917mm</xsl:when>
      <xsl:when test="$paper-type = 'C1'">648mm</xsl:when>
      <xsl:when test="$paper-type = 'C2'">458mm</xsl:when>
      <xsl:when test="$paper-type = 'C3'">324mm</xsl:when>
      <xsl:when test="$paper-type = 'C4'">229mm</xsl:when>
      <xsl:when test="$paper-type = 'C5'">162mm</xsl:when>
      <xsl:when test="$paper-type = 'C6'">114mm</xsl:when>
      <xsl:when test="$paper-type = 'C7'">81mm</xsl:when>
      <xsl:when test="$paper-type = 'C8'">57mm</xsl:when>
      <xsl:when test="$paper-type = 'C9'">40mm</xsl:when>
      <xsl:when test="$paper-type = 'C10'">28mm</xsl:when>
      <xsl:otherwise>8.5in</xsl:otherwise>
    </xsl:choose>
  </xsl:param>

  <xsl:param name="page-height-portrait">
    <xsl:choose>
      <xsl:when test="$paper-type = 'A4landscape'">210mm</xsl:when>
      <xsl:when test="$paper-type = 'USletter'">11in</xsl:when>
      <xsl:when test="$paper-type = 'USlandscape'">8.5in</xsl:when>
      <xsl:when test="$paper-type = '4A0'">2378mm</xsl:when>
      <xsl:when test="$paper-type = '2A0'">1682mm</xsl:when>
      <xsl:when test="$paper-type = 'A0'">1189mm</xsl:when>
      <xsl:when test="$paper-type = 'A1'">841mm</xsl:when>
      <xsl:when test="$paper-type = 'A2'">594mm</xsl:when>
      <xsl:when test="$paper-type = 'A3'">420mm</xsl:when>
      <xsl:when test="$paper-type = 'A4'">297mm</xsl:when>
      <xsl:when test="$paper-type = 'A5'">210mm</xsl:when>
      <xsl:when test="$paper-type = 'A6'">148mm</xsl:when>
      <xsl:when test="$paper-type = 'A7'">105mm</xsl:when>
      <xsl:when test="$paper-type = 'A8'">74mm</xsl:when>
      <xsl:when test="$paper-type = 'A9'">52mm</xsl:when>
      <xsl:when test="$paper-type = 'A10'">37mm</xsl:when>
      <xsl:when test="$paper-type = 'B0'">1414mm</xsl:when>
      <xsl:when test="$paper-type = 'B1'">1000mm</xsl:when>
      <xsl:when test="$paper-type = 'B2'">707mm</xsl:when>
      <xsl:when test="$paper-type = 'B3'">500mm</xsl:when>
      <xsl:when test="$paper-type = 'B4'">353mm</xsl:when>
      <xsl:when test="$paper-type = 'B5'">250mm</xsl:when>
      <xsl:when test="$paper-type = 'B6'">176mm</xsl:when>
      <xsl:when test="$paper-type = 'B7'">125mm</xsl:when>
      <xsl:when test="$paper-type = 'B8'">88mm</xsl:when>
      <xsl:when test="$paper-type = 'B9'">62mm</xsl:when>
      <xsl:when test="$paper-type = 'B10'">44mm</xsl:when>
      <xsl:when test="$paper-type = 'C0'">1297mm</xsl:when>
      <xsl:when test="$paper-type = 'C1'">917mm</xsl:when>
      <xsl:when test="$paper-type = 'C2'">648mm</xsl:when>
      <xsl:when test="$paper-type = 'C3'">458mm</xsl:when>
      <xsl:when test="$paper-type = 'C4'">324mm</xsl:when>
      <xsl:when test="$paper-type = 'C5'">229mm</xsl:when>
      <xsl:when test="$paper-type = 'C6'">162mm</xsl:when>
      <xsl:when test="$paper-type = 'C7'">114mm</xsl:when>
      <xsl:when test="$paper-type = 'C8'">81mm</xsl:when>
      <xsl:when test="$paper-type = 'C9'">57mm</xsl:when>
      <xsl:when test="$paper-type = 'C10'">40mm</xsl:when>
      <xsl:otherwise>11in</xsl:otherwise>
    </xsl:choose>
  </xsl:param>

  <xsl:param name="page-margin-top">1in</xsl:param>
  <xsl:param name="page-margin-bottom">1in</xsl:param>
  <xsl:param name="page-margin-left">1in</xsl:param>
  <xsl:param name="page-margin-right">1in</xsl:param>

  <!-- page header and footer -->
  <xsl:param name="page-header-margin">0.5in</xsl:param>
  <xsl:param name="page-footer-margin">0.5in</xsl:param>
  <xsl:param name="title-print-in-header">true</xsl:param>
  <xsl:param name="page-number-print-in-footer">true</xsl:param>

  <!-- text-align: justify | start -->
  <xsl:param name="text-align">justify</xsl:param>

  <!-- hyphenate: true | false -->
  <xsl:param name="hyphenate">true</xsl:param>

  <xsl:param name="font-family">serif</xsl:param>
  <xsl:param name="font-size">10pt</xsl:param>

  <!-- Prepended to the src attribute of the generated fo:external-graphic,
       unless the src of the XHTML img is absolute. 
       This value must be empty or must end with a '/'. -->
  <xsl:param name="img-src-path"></xsl:param>

  <!-- Specifies the ID of the element to be formatted.
       By default (empty string), it is the whole document. -->
  <xsl:param name="root-id"></xsl:param>

  <!--======================================================================
      Attribute Sets

      (Multiple definitions of an attribute set with the same expanded-name
       are merged.)
  =======================================================================-->

  <xsl:attribute-set name="root">
    <xsl:attribute name="font-family">
      <xsl:value-of select="$font-family"/>
    </xsl:attribute>
    <xsl:attribute name="font-size">
      <xsl:value-of select="$font-size"/>
    </xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="page-header">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
    <xsl:attribute name="color">gray</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="page-footer">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
    <xsl:attribute name="color">gray</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="h1">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="h2">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="h3">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="h4">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="h5">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="h6">
    <xsl:attribute name="font-family">sans-serif</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="pre">
    <xsl:attribute name="text-align">start</xsl:attribute>
    <xsl:attribute name="background-color">#F0F0F0</xsl:attribute>
    <xsl:attribute name="border">0.25pt solid #C0C0C0</xsl:attribute>
    <xsl:attribute name="padding">0.25em</xsl:attribute>
  </xsl:attribute-set>

  <xsl:attribute-set name="tt">
    <xsl:attribute name="font-size">0.83em</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="code">
    <xsl:attribute name="font-size">0.83em</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="kbd">
    <xsl:attribute name="font-size">0.83em</xsl:attribute>
  </xsl:attribute-set>
  <xsl:attribute-set name="samp">
    <xsl:attribute name="font-size">0.83em</xsl:attribute>
  </xsl:attribute-set>

  <!--======================================================================
      Support of parameter root-id (see above)
  =======================================================================-->

  <xsl:template match="/">
    <xsl:choose>
      <xsl:when test="$root-id != ''">
        <xsl:variable name="root" select="//*[@id = $root-id]"/>
        <xsl:choose>
          <xsl:when test="$root">
            <fo:root xsl:use-attribute-sets="root">
              <xsl:call-template name="make-layout-master-set"/>

              <fo:page-sequence master-reference="all-pages">
                <fo:static-content flow-name="page-header">
                  <fo:block space-before.conditionality="retain"
                            space-before="{$page-header-margin}"
                            xsl:use-attribute-sets="page-header">
                    <xsl:if test="$title-print-in-header = 'true'">
                      <xsl:value-of select="/html:html/html:head/html:title"/>
                    </xsl:if>
                  </fo:block>
                </fo:static-content>
                <fo:static-content flow-name="page-footer">
                  <fo:block space-after.conditionality="retain"
                            space-after="{$page-footer-margin}"
                            xsl:use-attribute-sets="page-footer">
                    <xsl:if test="$page-number-print-in-footer = 'true'">
                      <xsl:text>- </xsl:text>
                      <fo:page-number/>
                      <xsl:text> -</xsl:text>
                    </xsl:if>
                  </fo:block>
                </fo:static-content>
                <fo:flow flow-name="xsl-region-body">
                  <fo:block xsl:use-attribute-sets="body">
                    <xsl:apply-templates select="$root"/>
                  </fo:block>
                </fo:flow>
              </fo:page-sequence>
            </fo:root>
          </xsl:when>

          <xsl:otherwise>
            <xsl:message terminate="yes">
              <xsl:text>Do not find any element having </xsl:text>
              <xsl:value-of select="$root-id"/>
              <xsl:text> as its ID.</xsl:text>
            </xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>

      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
